@@ -1,13 +1,18 @@
 import logging
 import asyncio
 
-async def handle(cfg, metrics):
+async def handle(cfg, queue, metrics):
     """stdout output handler"""
     
     queries_prev = metrics.stats["total-queries"]
     while True:
         await asyncio.sleep(cfg["interval"])
         
+        # clear queue
+        for _ in range(queue.qsize()):
+            queue.get_nowait()
+            queue.task_done()
+ 
         if not cfg["cumulative"]:
             queries_prev = 0
         queries_cur = metrics.stats["total-queries"]
@@ -18,8 +23,8 @@ async def handle(cfg, metrics):
         msg.append( "%s QPS" % round(qps, 2))
         msg.append( "%s CLIENTS" % len(metrics.clients) )
         
-        msg.append( "%s IP4" % metrics.family.get("IP4", 0) )   
-        msg.append( "%s IP6" % metrics.family.get("IP6", 0) )
+        msg.append( "%s INET" % metrics.family.get("INET", 0) )   
+        msg.append( "%s INET6" % metrics.family.get("INET6", 0) )
 
         msg.append( "%s UDP" % metrics.proto.get("UDP", 0) )
         msg.append( "%s TCP" % metrics.proto.get("TCP", 0) )
