@@ -11,8 +11,10 @@ import ipaddress
 
 from datetime import datetime, timezone
 
-# python3 -m pip dnslib
-import dnslib
+# python3 -m pip dnspython
+import dns.rcode
+import dns.rdatatype
+import dns.message
 
 # wget https://raw.githubusercontent.com/dnstap/dnstap.pb/master/dnstap.proto
 # wget https://github.com/protocolbuffers/protobuf/releases/download/v3.13.0/protoc-3.13.0-linux-x86_64.zip
@@ -26,14 +28,12 @@ from dnstap_receiver import output_syslog
 from dnstap_receiver import output_tcp
 from dnstap_receiver import output_metrics
 
-DNSTAP_TYPE = { 1: 'AUTH_QUERY', 2: 'AUTH_RESPONSE',
-                3: 'RESOLVER_QUERY', 4: 'RESOLVER_RESPONSE',
-                5: 'CLIENT_QUERY', 6: 'CLIENT_RESPONSE',
-                7: 'FORWARDER_QUERY', 8: 'FORWARDER_RESPONSE',
-                9: 'STUB_QUERY', 10: 'STUB_RESPONSE',
-                11: 'TOOL_QUERY', 2: 'TOOL_RESPONSE' }
-DNSTAP_FAMILY = {1: 'IP4', 2: 'IP6'}
-DNSTAP_PROTO = {1: 'UDP', 2: 'TCP'}    
+class UnknownValue:
+    name = "-"
+
+DNSTAP_TYPE = dnstap_pb2._MESSAGE_TYPE.values_by_number
+DNSTAP_FAMILY = dnstap_pb2._SOCKETFAMILY.values_by_number
+DNSTAP_PROTO = dnstap_pb2._SOCKETPROTOCOL.values_by_number  
 
 # command line arguments definition
 parser = argparse.ArgumentParser()
@@ -47,6 +47,58 @@ parser.add_argument("-u", help="read dnstap payloads from unix socket")
 parser.add_argument('-v', action='store_true', help="verbose mode")   
 parser.add_argument("-c", help="external config file")   
 
+import dns.exception
+import dns.opcode
+import dns.flags
+
+# waiting fix with dnspython 2.1
+# will be removed in the future
+class _WireReader(dns.message._WireReader):
+    def read(self):
+        """issue fixed - waiting fix with dnspython 2.1"""
+        if self.parser.remaining() < 12:
+            raise dns.message.ShortHeader
+        (id, flags, qcount, ancount, aucount, adcount) = \
+            self.parser.get_struct('!HHHHHH')
+        factory = dns.message._message_factory_from_opcode(dns.opcode.from_flags(flags))
+        self.message = factory(id=id)
+        self.message.flags = flags
+        self.initialize_message(self.message)
+        self.one_rr_per_rrset = \
+            self.message._get_one_rr_per_rrset(self.one_rr_per_rrset)
+        self._get_question(dns.message.MessageSection.QUESTION, qcount)
+        
+        return self.message
+
+# waiting fix with dnspython 2.1
+# will be removed in the future
+def from_wire(wire, question_only=True):
+    """decode wire message - waiting fix with dnspython 2.1"""
+    raise_on_truncation=False
+    def initialize_message(message):
+        message.request_mac = b''
+        message.xfr = False
+        message.origin = None
+        message.tsig_ctx = None
+
+    reader = _WireReader(wire, initialize_message, question_only=question_only,
+                 one_rr_per_rrset=False, ignore_trailing=False,
+                 keyring=None, multi=False)
+    try:
+        m = reader.read()
+    except dns.exception.FormError:
+        if reader.message and (reader.message.flags & dns.flags.TC) and \
+           raise_on_truncation:
+            raise dns.message.Truncated(message=reader.message)
+        else:
+            raise
+    # Reading a truncated message might not have any errors, so we
+    # have to do this check here too.
+    if m.flags & dns.flags.TC and raise_on_truncation:
+        raise dns.message.Truncated(message=m)
+
+    return m
+    
 async def cb_ondnstap(dnstap_decoder, payload, cfg, queue, metrics):
     """on dnstap"""
     # decode binary payload
@@ -56,21 +108,19 @@ async def cb_ondnstap(dnstap_decoder, payload, cfg, queue, metrics):
     # filtering by dnstap identity ?
     tap_ident = dnstap_decoder.identity.decode()
     if not len(tap_ident):
-        tap_ident = "-"
+        tap_ident = UnknownValue.name
     if cfg["filter"]["dnstap-identities"] is not None:
         if re.match(cfg["filter"]["dnstap-identities"], dnstap_decoder.identity.decode()) is None:
             del dm
             return
             
-    tap = { "identity": tap_ident,
-            "query-name": "-", 
-            "query-type": "-", 
-            "source-ip": "-"}
+    tap = { "identity": tap_ident, "query-name": UnknownValue.name, 
+            "query-type": UnknownValue.name, "source-ip": UnknownValue.name}
     
     # decode type message
-    tap["message"] = DNSTAP_TYPE.get(dm.type, "-")
-    tap["protocol"] = DNSTAP_FAMILY.get(dm.socket_family, "-")
-    tap["transport"] = DNSTAP_PROTO.get(dm.socket_protocol, "-")
+    tap["message"] = DNSTAP_TYPE.get(dm.type, UnknownValue).name
+    tap["family"] = DNSTAP_FAMILY.get(dm.socket_family, UnknownValue).name
+    tap["protocol"] = DNSTAP_PROTO.get(dm.socket_protocol, UnknownValue).name
 
     # decode query address
     if len(dm.query_address) and dm.socket_family == 1:
@@ -83,23 +133,25 @@ async def cb_ondnstap(dnstap_decoder, payload, cfg, queue, metrics):
         
     # handle query message
     if (dm.type % 2 ) == 1 :
-        dnstap_parsed = dnslib.DNSRecord.parse(dm.query_message)
+        dnstap_parsed = from_wire(dm.query_message,
+                                  question_only=True)
         tap["length"] = len(dm.query_message)
         d1 = dm.query_time_sec +  (round(dm.query_time_nsec ) / 1000000000)
         tap["timestamp"] = datetime.fromtimestamp(d1, tz=timezone.utc).isoformat()
         
     # handle response message
     if (dm.type % 2 ) == 0 :
-        dnstap_parsed = dnslib.DNSRecord.parse(dm.response_message)
+        dnstap_parsed = from_wire(dm.response_message,
+                                  question_only=True)
         tap["length"] = len(dm.response_message)
         d2 = dm.response_time_sec + (round(dm.response_time_nsec ) / 1000000000) 
         tap["timestamp"] = datetime.fromtimestamp(d2, tz=timezone.utc).isoformat()
         
     # common params
-    if len(dnstap_parsed.questions):
-        tap["query-name"] = str(dnstap_parsed.questions[0].get_qname())
-        tap["query-type"] = dnslib.QTYPE[dnstap_parsed.questions[0].qtype]
-    tap["code"] = dnslib.RCODE[dnstap_parsed.header.rcode]
+    if len(dnstap_parsed.question):
+        tap["query-name"] = dnstap_parsed.question[0].name
+        tap["query-type"] = dns.rdatatype.to_text(dnstap_parsed.question[0].rdtype)
+    tap["code"] = dns.rcode.to_text(dnstap_parsed.rcode())
     
     # filtering by qname ?
     if cfg["filter"]["qname-regex"] is not None:
@@ -110,11 +162,7 @@ async def cb_ondnstap(dnstap_decoder, payload, cfg, queue, metrics):
     # update metrics 
     metrics.record_dnstap(dnstap=tap)
         
-    # finally add decoded tap message in queue for outputs
-    # except for metrics
-    if cfg["output"]["metrics"]["enable"]:
-        return
-        
+    # append the dnstap message to the queue
     queue.put_nowait(tap)
 
 async def cb_onconnect(reader, writer, cfg, queue, metrics):
@@ -226,15 +274,15 @@ class Metrics:
         """add dnstap message"""
         self.stats["total-queries"] += 1
 
-        if dnstap["transport"] not in self.proto:
-            self.proto[dnstap["transport"]] = 1
+        if dnstap["protocol"] not in self.proto:
+            self.proto[dnstap["protocol"]] = 1
         else:
-            self.proto[dnstap["transport"]] += 1
+            self.proto[dnstap["protocol"]] += 1
             
-        if dnstap["protocol"] not in self.family:
-            self.family[dnstap["protocol"]] = 1
+        if dnstap["family"] not in self.family:
+            self.family[dnstap["family"]] = 1
         else:
-            self.family[dnstap["protocol"]] += 1
+            self.family[dnstap["family"]] += 1
             
         if dnstap["query-name"] not in self.queries:
             self.queries[dnstap["query-name"]] = 1
@@ -307,22 +355,26 @@ def start_receiver():
     if cfg["output"]["syslog"]["enable"]:
         logging.debug("Output handler: syslog")
         loop.create_task(output_syslog.handle(cfg["output"]["syslog"], 
-                                              queue))
+                                              queue,
+                                              metrics))
         
     if cfg["output"]["tcp-socket"]["enable"]:
         logging.debug("Output handler: tcp")
         loop.create_task(output_tcp.handle(cfg["output"]["tcp-socket"],
-                                           queue))
+                                           queue,
+                                           metrics))
 
     if cfg["output"]["stdout"]["enable"]:
         logging.debug("Output handler: stdout")
         loop.create_task(output_stdout.handle(cfg["output"]["stdout"],
-                                              queue))
+                                              queue,
+                                              metrics))
 
     
     if cfg["output"]["metrics"]["enable"]:
         logging.debug("Output handler: metrics")
         loop.create_task(output_metrics.handle(cfg["output"]["metrics"],
+                                              queue,
                                               metrics))
                                               
 
