@@ -11,8 +11,8 @@ def convert_dnstap(fmt, tapmsg):
         msg_list.append("%s" % tapmsg["code"]) 
         msg_list.append("%s" % tapmsg["source-ip"])
         msg_list.append("%s" % tapmsg["source-port"])
+        msg_list.append("%s" % tapmsg["family"])
         msg_list.append("%s" % tapmsg["protocol"])
-        msg_list.append("%s" % tapmsg["transport"])
         msg_list.append("%sb" % tapmsg["length"])
         msg_list.append("%s" % tapmsg["query-name"])
         msg_list.append("%s" % tapmsg["query-type"])
